# -*- coding: utf-8 -*-
from odoo.addons.saas_worker.compat import route, request

import hmac
import base64
import hashlib
import logging
import time
from markupsafe import Markup
import werkzeug.utils
import werkzeug.urls
import re

from odoo.addons.saas_worker.const import NIGHTLY_SECRET, TEST
from odoo.addons.saas_worker.util import ip2coordinates

from odoo.addons.website_crm.controllers.website_form import WebsiteForm

_logger = logging.getLogger(__name__)


class ContactController(WebsiteForm):
    def _get_location(self, req, ip_address=None):
        """return location from odoo request 'req'"""
        country_id = None
        location = {}
        if not ip_address:
            ip_address = req.httprequest.environ['REMOTE_ADDR']
        if ip_address:
            coordinates = ip2coordinates(ip_address)
            country_code = coordinates['country_code']
            country = req.env['res.country'].sudo().search([('code', '=', country_code.upper())], limit=1)
            country_id = country.id
            if coordinates.get('city'):
                location['city'] = coordinates['city']
            if coordinates.get('longitude'):
                location['partner_longitude'] = float(coordinates['longitude'])
            if coordinates.get('latitude'):
                location['partner_latitude'] = float(coordinates['latitude'])
            if coordinates.get('region'):
                state = country.state_ids.filtered(lambda x: x.code == coordinates.get('region'))
                location['state_id'] = state and state[0].id

        return country_id, location

    def insert_record(self, request, model, values, custom, meta=None):
        if model.model != 'crm.lead':
            return super().insert_record(request, model, values, custom, meta=meta)

        kwargs = request.params
        # identify the partner and his location
        country_id, location = self._get_location(request)
        partner = request.env['res.partner'].sudo().search([
            ('email', '=', values['email_from']),
            ('type', '!=', 'private')], limit=1)

        context_all_values = request.context.copy()
        context_all_values['all_lead_values'] = values
        context_all_values['all_lead_kwargs'] = kwargs

        # create the custom lead
        Lead = request.env['crm.lead'].sudo().with_context(context_all_values)
        if not hasattr(Lead, 'create_lead'):
            _logger.warning('Need Openerp_enteprise to create a lead')
            return 0

        lead_id = Lead.create_lead(
            text=values['description'],
            email=values['email_from'],
            contact_name=values['contact_name'],
            company=values.get('partner_name', values['contact_name']),  # From download dont ask company
            phone=values.get('phone', ''),
            partner_id=partner.id,
            scope=values.get('force_scope') or kwargs.get('force_scope'),
            plan=values.get('plan') or kwargs.get('plan', values.get('name', '-*-')),
            country_id=country_id,
            location=location
        )

        visitor = request.env['website.visitor']._get_visitor_from_request().sudo()
        if visitor and lead_id:
            vals = {'lead_ids': [(4, lead_id.id)]}
            if not visitor.lead_ids and not visitor.partner_id:
                vals['name'] = lead_id.contact_name
            visitor.write(vals)

        return lead_id.id

    def get_download_url(self, version, platform, code=False):

        def gen_token(version, fmt, code=False):
            if code:
                subscription = request.env['sale.order'].sudo()._get_most_valid_subscription(code)
                has_download = bool(subscription) and subscription.enterprise_has_download
            is_authorized = code and has_download or request.env.user.has_enterprise()

            # Only require authorization for Enterprise Edition.
            # Tokens are version-specific and can't be reused for other packages.
            if version.endswith('e') and not is_authorized:
                return ''

            max_timestamp = int(time.time()) + 3600
            message = f'{max_timestamp}.{version}.{fmt}'.encode()
            digest = hmac.new(NIGHTLY_SECRET, message, hashlib.sha256).digest()
            sig = base64.urlsafe_b64encode(digest)
            payload = base64.urlsafe_b64encode(message + b'.' + sig)
            return werkzeug.urls.url_encode({'payload': payload})

        if platform == 'win':
            # legacy alias
            platform = 'exe'

        if not re.match(r'^[0-9]{2}e?$', version) or platform not in ('exe', 'deb', 'rpm', 'src'):
            return 'There is no matching version for your selection at the moment.'

        # generate token if code is valid.
        token = '?' + gen_token(version, platform, code)

        url = 'https://download.odoocdn.com/download/{version}/{platform}'.format(version=version, platform=platform)

        if TEST:
            _logger.warning("The real download URL is %s%s" % (url, token))
            token = "?payload=from_test_see_log"

        ret = """
            <a target='_blank' href='{0}' class='start_download'>Click here</a> if it has not started after few seconds.
            <script>document.addEventListener("DOMContentLoaded", function(event) {{
                setTimeout(function() {{ window.location.assign('{0}'); }}, 2000)
            }});</script>""".format(url + token)
        return Markup(ret)

    @route(['/download/check_subscription'], type="json", auth="public", readonly=True)
    def can_download_check(self, code):
        result = "success"
        subscription = request.env['sale.order'].sudo()._get_most_valid_subscription(code.strip())
        if not subscription:
            result = "oe_download_invalid_code"
        elif subscription:
            if subscription.subscription_state not in ['3_progress', '4_paused']:
                result = "oe_download_invalid_status"
            elif not subscription.enterprise_has_download:
                result = "oe_download_not_allowed"
        return result

    @route(['/page/download'], type="http", auth="public", website=True, sitemap=True, readonly=True)
    def download(self, code=False, **kwargs):
        params = {
            'code': code,
            'has_valid_code': False,
            'has_enterprise': request.env.user.has_enterprise(),
        }
        if code:
            contract = request.env['sale.order'].sudo()._get_most_valid_subscription(code)
            if contract and contract.enterprise_has_download:
                params['has_valid_code'] = contract.enterprise_has_download
                params['subscription_name'] = contract.display_name
        return request.render('openerp_website.download', params)

    @route(['/thanks/download'], type='http', auth="public", website=True, sitemap=False)
    def thanks_download(self, platform_version=False, code=False, **kwargs):
        download_page = '/page/download'
        lead_id = None
        _REQUIRED = ['contact_name', 'email_from']

        # Prevent crash at refresh or because of invalid values as params
        lead_id = platform = version = False
        if platform_version and re.match("^(win|deb|rpm|src)_[0-9]{2}e?$", platform_version):
            [platform, version] = platform_version.split('_')
        if not platform_version or not (platform and version):
            return request.redirect(download_page)

        if not request.httprequest.cookies.get('download_form') and not code and not request.env.user.has_group('base.group_user'):
            error = set(field for field in _REQUIRED if not kwargs.get(field))
            if error:
                return request.render("openerp_website.download", dict(kwargs, error=error))
            # RETRO-compatibility - simulation of website_form / old contactus - TO REMOVE when we will refactor scoring rules
            kwargs['description'] += '\n\nCustom Fields:\nplatform_version: %s\nplan: %s\ncompany_size: %s\n' % (platform_version, kwargs.get('plan'), kwargs.get('company_size'))
            model_record = request.env['ir.model'].sudo().search([('model', '=', 'crm.lead')])
            lead_id = self.insert_record(request, model_record, kwargs, {})

        values = {
            'version': version,
            'platform': platform,
            'links': self.get_download_url(version, platform, code),
            'new_lead': lead_id,
        }
        response = request.render('openerp_website.thanks_download', values)

        if lead_id:
            response.set_cookie('download_form', 'true')
        return response
